package cn.com.qmth.scancentral.service.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import cn.com.qmth.scancentral.vo.SubjectConfigVo;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.core.collection.PageResult;
import com.qmth.boot.core.exception.ParameterException;

import cn.com.qmth.scancentral.bean.User;
import cn.com.qmth.scancentral.dao.MarkSiteDao;
import cn.com.qmth.scancentral.entity.MarkSiteEntity;
import cn.com.qmth.scancentral.service.MarkSiteService;
import cn.com.qmth.scancentral.util.PageUtil;
import cn.com.qmth.scancentral.vo.marksite.MarkSiteDomain;
import cn.com.qmth.scancentral.vo.marksite.MarkSitePageQuery;
import cn.com.qmth.scancentral.vo.marksite.MarkSitePageVo;

@Service
public class MarkSiteServiceImpl extends ServiceImpl<MarkSiteDao, MarkSiteEntity> implements MarkSiteService {

    @Override
    public PageResult<MarkSitePageVo> findMarkSitePage(MarkSitePageQuery query) {
        IPage<MarkSitePageVo> iPage = baseMapper
                .findMarkSitePage(new Page<>(query.getPageNumber(), query.getPageSize()), query);
        return PageUtil.of(iPage);
    }

    private MarkSiteEntity findByExamSubjectPaperType(Long examId, String subjectCode, String paperType) {
        QueryWrapper<MarkSiteEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<MarkSiteEntity> lw = wrapper.lambda();
        lw.eq(MarkSiteEntity::getExamId, examId);
        lw.eq(MarkSiteEntity::getSubjectCode, subjectCode);
        lw.eq(MarkSiteEntity::getPaperType, paperType);
        return this.getOne(wrapper);
    }

    @Transactional
    @Override
    public void saveMarkSite(MarkSiteDomain domain, User user) {
        MarkSiteEntity old = findByExamSubjectPaperType(domain.getExamId(), domain.getSubjectCode(),
                domain.getPaperType());
        if ((domain.getId() == null && old != null)
                || (domain.getId() != null && old != null && !domain.getId().equals(old.getId()))) {
            throw new ParameterException("评卷点信息已存在");
        }
        if (domain.getId() != null) {
            // 修改
            MarkSiteEntity e = this.getById(domain.getId());
            if (e == null) {
                throw new ParameterException("评卷点不存在");
            }

            LambdaUpdateWrapper<MarkSiteEntity> updateWrapper = new LambdaUpdateWrapper<>();
            updateWrapper.set(MarkSiteEntity::getUpdateTime, System.currentTimeMillis());
            updateWrapper.set(MarkSiteEntity::getUpdaterId, user.getId());
            updateWrapper.set(MarkSiteEntity::getSubjectCode, domain.getSubjectCode());
            updateWrapper.set(MarkSiteEntity::getPaperType, domain.getPaperType());
            updateWrapper.set(MarkSiteEntity::getOddNumber, domain.getOddNumber());
            updateWrapper.set(MarkSiteEntity::getEvenNumber, domain.getEvenNumber());
            updateWrapper.eq(MarkSiteEntity::getId, domain.getId());
            this.update(updateWrapper);

        } else {

            // 新增
            MarkSiteEntity e = new MarkSiteEntity();
            e.setExamId(domain.getExamId());
            e.setSubjectCode(domain.getSubjectCode());
            e.setPaperType(domain.getPaperType());
            e.setOddNumber(domain.getOddNumber());
            e.setEvenNumber(domain.getEvenNumber());
            this.save(e);
        }
    }

    @Override
    public List<MarkSiteEntity> findByExamAndSubject(Long examId, String subejctCode) {
        QueryWrapper<MarkSiteEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<MarkSiteEntity> lw = wrapper.lambda();
        lw.eq(MarkSiteEntity::getExamId, examId);
        lw.eq(MarkSiteEntity::getSubjectCode, subejctCode);
        return this.list(wrapper);
    }

    @Override
    public List<SubjectConfigVo> findByExam(Long examId) {
        QueryWrapper<MarkSiteEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<MarkSiteEntity> lw = wrapper.lambda();
        lw.eq(MarkSiteEntity::getExamId, examId);
        List<MarkSiteEntity> list = this.list(wrapper);
        List<SubjectConfigVo> result = new ArrayList<>();
        Map<String, List<MarkSiteEntity>> map = list.stream().collect(Collectors.groupingBy(MarkSiteEntity::getSubjectCode));
        for (String subjectCode: map.keySet()) {
            SubjectConfigVo vo = new SubjectConfigVo();
            vo.setCode(subjectCode);
            vo.setPaperTypeBarcodeContent(map.get(subjectCode).stream().map(MarkSiteEntity::getPaperType).collect(Collectors.toList()));
            result.add(vo);
        }
        return result;
    }

    @Override
    @Transactional
    public void saveByExamIdAndSubjectCode(Long examId, List<SubjectConfigVo> paperTypeBarcodeContent) {
        List<MarkSiteEntity> saveList = new ArrayList<>();
        for (SubjectConfigVo vo: paperTypeBarcodeContent) {
            List<MarkSiteEntity> oldList = this.findByExamAndSubject(examId,vo.getCode());
            List<MarkSiteEntity> deleteList = oldList.stream().filter(e->!vo.getPaperTypeBarcodeContent().contains(e.getPaperType())).collect(Collectors.toList());
            this.removeByIds(deleteList);
            List<String> existPaperTypeList = oldList.stream().filter(e->vo.getPaperTypeBarcodeContent().contains(e.getPaperType())).map(MarkSiteEntity::getPaperType).collect(Collectors.toList());
            for (String paperType: vo.getPaperTypeBarcodeContent()) {
                if(existPaperTypeList.contains(paperType)){
                    continue;
                }
                MarkSiteEntity e = new MarkSiteEntity();
                e.setExamId(examId);
                e.setSubjectCode(vo.getCode());
                e.setPaperType(paperType);
                saveList.add(e);
            }
        }
        this.saveBatch(saveList);
    }
}
